var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import baseLogger from './logger';
import EventEmitter from './EventEmitter';
import ResourceStore from './ResourceStore';
import Translator from './Translator';
import LanguageUtils from './LanguageUtils';
import PluralResolver from './PluralResolver';
import Interpolator from './Interpolator';
import BackendConnector from './BackendConnector';
import CacheConnector from './CacheConnector';
import { get as getDefaults, transformOptions } from './defaults';
import postProcessor from './postProcessor';

import * as compat from './compatibility/v1';

var I18n = function (_EventEmitter) {
  _inherits(I18n, _EventEmitter);

  function I18n() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    var callback = arguments[1];

    _classCallCheck(this, I18n);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(I18n).call(this));

    _this.options = transformOptions(options);
    _this.services = {};
    _this.logger = baseLogger;
    _this.modules = {};

    if (callback && !_this.isInitialized) _this.init(options, callback);
    return _this;
  }

  _createClass(I18n, [{
    key: 'init',
    value: function init(options, callback) {
      var _this2 = this;

      if (typeof options === 'function') {
        callback = options;
        options = {};
      }
      if (!options) options = {};

      if (options.compatibilityAPI === 'v1') {
        this.options = _extends({}, getDefaults(), transformOptions(compat.convertAPIOptions(options)), {});
      } else if (options.compatibilityJSON === 'v1') {
        this.options = _extends({}, getDefaults(), transformOptions(compat.convertJSONOptions(options)), {});
      } else {
        this.options = _extends({}, getDefaults(), this.options, transformOptions(options));
      }
      if (!callback) callback = function callback() {};

      function createClassOnDemand(ClassOrObject) {
        if (!ClassOrObject) return;
        if (typeof ClassOrObject === 'function') return new ClassOrObject();
        return ClassOrObject;
      }

      // init services
      if (!this.options.isClone) {
        if (this.modules.logger) {
          baseLogger.init(createClassOnDemand(this.modules.logger), this.options);
        } else {
          baseLogger.init(null, this.options);
        }

        var lu = new LanguageUtils(this.options);
        this.store = new ResourceStore(this.options.resources, this.options);

        var s = this.services;
        s.logger = baseLogger;
        s.resourceStore = this.store;
        s.resourceStore.on('added removed', function (lng, ns) {
          s.cacheConnector.save();
        });
        s.languageUtils = lu;
        s.pluralResolver = new PluralResolver(lu, { prepend: this.options.pluralSeparator, compatibilityJSON: this.options.compatibilityJSON });
        s.interpolator = new Interpolator(this.options);

        s.backendConnector = new BackendConnector(createClassOnDemand(this.modules.backend), s.resourceStore, s, this.options);
        // pipe events from backendConnector
        s.backendConnector.on('*', function (event) {
          for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
          }

          _this2.emit.apply(_this2, [event].concat(args));
        });

        s.backendConnector.on('loaded', function (loaded) {
          s.cacheConnector.save();
        });

        s.cacheConnector = new CacheConnector(createClassOnDemand(this.modules.cache), s.resourceStore, s, this.options);
        // pipe events from backendConnector
        s.cacheConnector.on('*', function (event) {
          for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
            args[_key2 - 1] = arguments[_key2];
          }

          _this2.emit.apply(_this2, [event].concat(args));
        });

        if (this.modules.languageDetector) {
          s.languageDetector = createClassOnDemand(this.modules.languageDetector);
          s.languageDetector.init(s, this.options.detection, this.options);
        }

        this.translator = new Translator(this.services, this.options);
        // pipe events from translator
        this.translator.on('*', function (event) {
          for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
            args[_key3 - 1] = arguments[_key3];
          }

          _this2.emit.apply(_this2, [event].concat(args));
        });
      }

      // append api
      var storeApi = ['getResource', 'addResource', 'addResources', 'addResourceBundle', 'removeResourceBundle', 'hasResourceBundle', 'getResourceBundle'];
      storeApi.forEach(function (fcName) {
        _this2[fcName] = function () {
          return this.store[fcName].apply(this.store, arguments);
        };
      });

      // TODO: COMPATIBILITY remove this
      if (this.options.compatibilityAPI === 'v1') compat.appendBackwardsAPI(this);

      this.changeLanguage(this.options.lng, function (err, t) {
        _this2.emit('initialized', _this2.options);
        _this2.logger.log('initialized', _this2.options);

        callback(err, t);
      });

      return this;
    }
  }, {
    key: 'loadResources',
    value: function loadResources(callback) {
      var _this3 = this;

      if (!callback) callback = function callback() {};

      if (!this.options.resources) {
        var _ret = function () {
          if (_this3.language && _this3.language.toLowerCase() === 'cimode') return {
              v: callback()
            }; // avoid loading resources for cimode

          var toLoad = [];

          var append = function append(lng) {
            var lngs = _this3.services.languageUtils.toResolveHierarchy(lng);
            lngs.forEach(function (l) {
              if (toLoad.indexOf(l) < 0) toLoad.push(l);
            });
          };

          append(_this3.language);

          if (_this3.options.preload) {
            _this3.options.preload.forEach(function (l) {
              append(l);
            });
          }

          _this3.services.cacheConnector.load(toLoad, _this3.options.ns, function () {
            _this3.services.backendConnector.load(toLoad, _this3.options.ns, callback);
          });
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      } else {
        callback(null);
      }
    }
  }, {
    key: 'use',
    value: function use(module) {
      if (module.type === 'backend') {
        this.modules.backend = module;
      }

      if (module.type === 'cache') {
        this.modules.cache = module;
      }

      if (module.type === 'logger' || module.log && module.warn && module.warn) {
        this.modules.logger = module;
      }

      if (module.type === 'languageDetector') {
        this.modules.languageDetector = module;
      }

      if (module.type === 'postProcessor') {
        postProcessor.addPostProcessor(module);
      }

      return this;
    }
  }, {
    key: 'changeLanguage',
    value: function changeLanguage(lng, callback) {
      var _this4 = this;

      var done = function done(err) {
        if (lng) {
          _this4.emit('languageChanged', lng);
          _this4.logger.log('languageChanged', lng);
        }

        if (callback) callback(err, function () {
          for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
            args[_key4] = arguments[_key4];
          }

          return _this4.t.apply(_this4, args);
        });
      };

      if (!lng && this.services.languageDetector) lng = this.services.languageDetector.detect();

      if (lng) {
        this.language = lng;
        this.languages = this.services.languageUtils.toResolveHierarchy(lng);

        this.translator.changeLanguage(lng);

        if (this.services.languageDetector) this.services.languageDetector.cacheUserLanguage(lng);
      }

      this.loadResources(function (err) {
        done(err);
      });
    }
  }, {
    key: 'getFixedT',
    value: function getFixedT(lng, ns) {
      var _this5 = this;

      var fixedT = function fixedT(key, options) {
        options = options || {};
        options.lng = options.lng || fixedT.lng;
        options.ns = options.ns || fixedT.ns;
        return _this5.t(key, options);
      };
      fixedT.lng = lng;
      fixedT.ns = ns;
      return fixedT;
    }
  }, {
    key: 't',
    value: function t() {
      return this.translator && this.translator.translate.apply(this.translator, arguments);
    }
  }, {
    key: 'exists',
    value: function exists() {
      return this.translator && this.translator.exists.apply(this.translator, arguments);
    }
  }, {
    key: 'setDefaultNamespace',
    value: function setDefaultNamespace(ns) {
      this.options.defaultNS = ns;
    }
  }, {
    key: 'loadNamespaces',
    value: function loadNamespaces(ns, callback) {
      var _this6 = this;

      if (!this.options.ns) return callback && callback();
      if (typeof ns === 'string') ns = [ns];

      ns.forEach(function (n) {
        if (_this6.options.ns.indexOf(n) < 0) _this6.options.ns.push(n);
      });

      this.loadResources(callback);
    }
  }, {
    key: 'loadLanguages',
    value: function loadLanguages(lngs, callback) {
      if (typeof lngs === 'string') lngs = [lngs];
      this.options.preload = this.options.preload ? this.options.preload.concat(lngs) : lngs;

      this.loadResources(callback);
    }
  }, {
    key: 'dir',
    value: function dir(lng) {
      if (!lng) lng = this.language;

      var ltrLngs = ['ar', 'shu', 'sqr', 'ssh', 'xaa', 'yhd', 'yud', 'aao', 'abh', 'abv', 'acm', 'acq', 'acw', 'acx', 'acy', 'adf', 'ads', 'aeb', 'aec', 'afb', 'ajp', 'apc', 'apd', 'arb', 'arq', 'ars', 'ary', 'arz', 'auz', 'avl', 'ayh', 'ayl', 'ayn', 'ayp', 'bbz', 'pga', 'he', 'iw', 'ps', 'pbt', 'pbu', 'pst', 'prp', 'prd', 'ur', 'ydd', 'yds', 'yih', 'ji', 'yi', 'hbo', 'men', 'xmn', 'fa', 'jpr', 'peo', 'pes', 'prs', 'dv', 'sam'];

      return ltrLngs.indexOf(this.services.languageUtils.getLanguagePartFromCode(lng)) ? 'ltr' : 'rtl';
    }
  }, {
    key: 'createInstance',
    value: function createInstance() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var callback = arguments[1];

      return new I18n(options, callback);
    }
  }, {
    key: 'cloneInstance',
    value: function cloneInstance() {
      var _this7 = this;

      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var callback = arguments[1];

      var clone = new I18n(_extends({}, options, this.options, { isClone: true }), callback);
      var membersToCopy = ['store', 'translator', 'services', 'language'];
      membersToCopy.forEach(function (m) {
        clone[m] = _this7[m];
      });

      return clone;
    }
  }]);

  return I18n;
}(EventEmitter);

export default new I18n();